--[[
	Copyright (c) 2013 David Young dayoung@goliathdesigns.com

	This software is provided 'as-is', without any express or implied
	warranty. In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
	claim that you wrote the original software. If you use this software
	in a product, an acknowledgement in the product documentation would be
	appreciated but is not required.

	2. Altered source versions must be plainly marked as such, and must not be
	misrepresented as being the original software.

	3. This notice may not be removed or altered from any source
	distribution.

**** Code rewritten for use in the Skyline Game Engine by SpinalSoft (AU) 2017 ****
]]

function actionNew ( name, tAct, objID )
	local act_Status = {
		RUNNING = "RUNNING",
		TERMINATED = "TERMINATED",
		UNINITIALIZED = "UNINITIALIZED"
	}

	actType = "Action"

	local self = {
		act_cleanUpFunction_ = tAct.clean,
		act_initializeFunction_ = tAct.init,
		act_updateFunction_ = tAct.upd,
		act_name_ = name or "",
		act_status_ = act_Status.UNINITIALIZED,
		act_type_ = actType,
		act_objID_ = objID
	}

	local act_cleanUp = function ( self, objID )
		if ( self.act_status_ == act_Status.TERMINATED ) then
			if ( self.act_cleanUpFunction_ ) then
				self.act_cleanUpFunction_ ( self, self.act_objID_ )
			end
		end

		self.act_status_ = act_Status.UNINITIALIZED
	end

	local act_initialize = function ( self, objID )
		if ( self.act_status_ == act_Status.UNINITIALIZED ) then
			if ( self.act_initializeFunction_ ) then
				self.act_initializeFunction_ ( self, self.act_objID_ )
			end
		end

		self.act_status_ = act_Status.RUNNING
	end

	local act_update = function ( self, objID, deltaTimeInMillis )
		if ( self.act_status_ == act_Status.TERMINATED ) then
			return act_Status.TERMINATED
		elseif ( self.act_status_ == act_Status.RUNNING ) then
			if ( self.act_updateFunction_ ) then
				self.act_status_ = self.act_updateFunction_ ( self, self.act_objID_, deltaTimeInMillis )

				if ( not self.act_status_ ) then
					sky.lprint ( "ACTION ERROR - self.status_ is FALSE" )
				end
			else
				self.act_status_ = act_Status.TERMINATED
			end
		end

		return self.act_status_
	end

	local act_getType = function ( )
		return self.act_type_
	end

	return {
		act_cleanUpFunction_ = tAct.clean,
		act_initializeFunction_ = tAct.init,
		act_updateFunction_ = tAct.upd,
		act_name_ = name or "",
		act_status_ = act_Status.UNINITIALIZED,
		act_type_ = actType,
		act_objID_ = objID,

		ACT_CleanUp = act_cleanUp,
		ACT_Initialize = act_initialize,
		ACT_Update = act_update,
		ACT_GetType = act_getType
	}
end