--[[
	Copyright (c) 2013 David Young dayoung@goliathdesigns.com

	This software is provided 'as-is', without any express or implied
	warranty. In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented you must not
	claim that you wrote the original software. If you use this software
	in a product, an acknowledgement in the product documentation would be
	appreciated but is not required.

	2. Altered source versions must be plainly marked as such, and must not be
	misrepresented as being the original software.

	3. This notice may not be removed or altered from any source
	distribution.

**** Code rewritten for use in the Skyline Game Engine by SpinalSoft (AU) 2017 ****
]]

function dbNew ( )
	dbType = "DecisionBranch"

	local self = {
		db_children_ = { },
		db_evaluator_ = nil,
		db_type_ = dbType
	}

	local db_addChild = function ( self, child, index )
		index = index or ( #self.db_children_ + 1 )
		table.insert ( self.db_children_, index, child )
	end

	local db_evaluate = function ( self )
		if ( not self.db_evaluator_ ) then
			sky.lprint ( "DECISION BRANCH ERROR - self.evaluator_ returning FALSE" )
		end

		local eval = self.db_evaluator_ ( )

		if ( type ( eval ) ~= "number" ) then
			sky.lprint ( "DECISION BRANCH ERROR - eval MUST return a Number" )
		end

		if ( eval > #self.db_children_ ) then
			sky.lprint ( "DECISION BRANCH ERROR - eval MUST choose a Child to execute" )
		end

		local choice = self.db_children_[eval]

		if ( type ( choice ) ~= "function" ) then
			if ( choice.db_type_ == dbType ) then
				return choice:DB_Evaluate ( )
			else
				return choice
			end
		else
			return choice
		end
	end

	local db_setEvaluator = function ( self, evaluator )
		if ( type ( evaluator ) ~= "function" ) then
			sky.lprint ( "DECISION BRANCH ERROR - evaluator MUST be a Function" )
		end

		self.db_evaluator_ = evaluator
	end

	local db_getType = function ( self )
		return self.db_type_
	end

	return {
		db_children_ = { },
		db_evaluator_ = nil,
		db_type_ = dbType,

		DB_AddChild = db_addChild,
		DB_Evaluate = db_evaluate,
		DB_SetEvaluator = db_setEvaluator,
		DB_GetType = db_getType
	}
end