--[[
	Copyright (c) 2013 David Young dayoung@goliathdesigns.com

	This software is provided 'as-is', without any express or implied
	warranty. In no event will the authors be held liable for any damages
	arising from the use of this software.

	Permission is granted to anyone to use this software for any purpose,
	including commercial applications, and to alter it and redistribute it
	freely, subject to the following restrictions:

	1. The origin of this software must not be misrepresented; you must not
	claim that you wrote the original software. If you use this software
	in a product, an acknowledgement in the product documentation would be
	appreciated but is not required.

	2. Altered source versions must be plainly marked as such, and must not be
	misrepresented as being the original software.

	3. This notice may not be removed or altered from any source
	distribution.

**** Code rewritten for use in the Skyline Game Engine by SpinalSoft (AU) 2017 ****
]]

sky.include "action.lua"
sky.include "finitestate.lua"
sky.include "finitestatetransition.lua"

function fsmNew ( objID )
	local self = {
		fsm_currentState_ = nil,
		fsm_states_ = {},
		fsm_transitions_ = {},
		fsm_objID_ = objID
	}

	fsm_evaluateTransitions = function ( self, transitions )
		for index = 1, #transitions do
			if ( transitions[index].fst_evaluator_ ( objID ) ) then
				return transitions[index].fst_toStateName_
			end
		end
	end

	local fsm_addState = function ( self, name, action )
		self.fsm_states_[name] = fsNew ( name, action )
	end

	local fsm_addTransition = function ( self, fromStateName, toStateName, evaluator )
		if ( fsm_containsState ( self, fromStateName ) and fsm_containsState ( self, toStateName ) ) then
			if ( self.fsm_transitions_[fromStateName] == nil ) then
				self.fsm_transitions_[fromStateName] = { }
			end

			table.insert( self.fsm_transitions_[fromStateName], fstNew ( toStateName, evaluator ) )
		end
	end

	fsm_containsState = function ( self, stateName )
		return self.fsm_states_[stateName] ~= nil
	end

	local fsm_containsTransition = function ( self, fromStateName, toStateName )
		return self.fsm_transitions_[fromStateName] ~= nil and self.fsm_transitions_[fromStateName][toStateName] ~= nil
	end

	local fsm_getCurrentStateName = function ( self )
		if ( self.fsm_currentState_ ) then
			return self.fsm_currentState_.fs_name_
		end
	end

	local fsm_getCurrentStateStatus = function ( self )
		if ( self.fsm_currentState_ ) then
			return self.fsm_currentState_.fs_action_.act_status_
		end
	end

	local fsm_setState = function ( self, stateName )
		if ( fsm_containsState ( self, stateName ) ) then
			if ( self.fsm_currentState_ ) then
				self.fsm_currentState_.fs_action_:ACT_CleanUp ( objID )
			end

			self.fsm_currentState_ = self.fsm_states_[stateName]
			self.fsm_currentState_.fs_action_:ACT_Initialize ( objID )
			self.fsm_currentState_.fs_action_.act_status_ = "TERMINATED"
		end
	end

	local fsm_update = function ( self, deltaTimeInMillis )
		if ( self.fsm_currentState_ ) then
			local status = fsm_getCurrentStateStatus ( self )

			if ( status == "RUNNING" ) then
				self.fsm_currentState_.fs_action_.act_status_ = self.fsm_currentState_.fs_action_:ACT_Update ( objID, deltaTimeInMillis )
			elseif ( status == "TERMINATED" ) then
				local toStateName = fsm_evaluateTransitions ( self, self.fsm_transitions_[self.fsm_currentState_.fs_name_] )

				if ( self.fsm_states_[toStateName] ~= nil ) then
					self.fsm_currentState_.fs_action_:ACT_CleanUp ( objID )
					self.fsm_currentState_ = self.fsm_states_[toStateName]
					self.fsm_currentState_.fs_action_:ACT_Initialize ( objID )
					self.fsm_currentState_.fs_action_.act_status_ = "RUNNING"
				end
			end
		end
	end

	return {
		fsm_currentState_ = nil,
		fsm_states_ = {},
		fsm_transitions_ = {},
		fsm_objID_ = objID,

		FSM_AddState = fsm_addState,
		FSM_AddTransition = fsm_addTransition,
		FSM_ContainsState = fsm_containsState,
		FSM_ContainsTransition = fsm_containsTransition,
		FSM_GetCurrentStateName = fsm_getCurrentStateName,
		FSM_GetCurrentStateStatus = fsm_getCurrentStateStatus,
		FSM_SetState = fsm_setState,
		FSM_Update = fsm_update
	}
end